#ifndef MY_I2C_H
#define MY_I2C_H

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//       ,            !!
//     I2C.                                          !!
//  SDA - PORTD.1  SCL - PORTD.0                                       !!
//													   !!
//   #define SDA_PIN     1									   !!
//   #define SCL_PIN     0									   !!
//   #define I2C_PORTIN  PIND									   !!
//   #define I2C_PORT    PORTD								   !!
//   #define I2C_DDR     DDRD									   !!
//   #include <My_I2C.h>								         !!
//													   !!
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include <delay.h>

#define ACK         1
#define NACK        0


/*****************************************************************************
*
*     : SDA_SET
*
*     : .
*
*    :    SDA (0  1).
*
*    :         
*             - SDA.
*
*             
*****************************************************************************/ 

void SDA_SET(unsigned char status)
{
    if(status == 1)
    {
        I2C_DDR.SDA_PIN = 0;
        I2C_PORT.SDA_PIN = 0;  
    }
        else
        {
            I2C_DDR.SDA_PIN = 1;
            I2C_PORT.SDA_PIN = 0;
        }
  
  delay_us(5);
}     



/*****************************************************************************
*
*     : SCL_SET
*
*     : .
*
*    :    SCL (0  1).
*
*    :       -
*              - SCL.
*
*             
*****************************************************************************/

void SCL_SET(unsigned char status)
{
    if(status == 1)
    {
        I2C_DDR.SCL_PIN = 0;
        I2C_PORT.SCL_PIN = 0;
    }   
        else
        {
            I2C_DDR.SCL_PIN = 1;
            I2C_PORT.SCL_PIN = 0;
        }
     
  delay_us(5);
} 


                  
/*****************************************************************************
*
*     : I2C_Start
*
*     : .
*
*    : .
*
*    :    START,    .
*		
*                               |----------- - - - 
*   		                    | SCL
*		      _ _ _ ________| 
*
*                          |--------|
*                          |        | SDA 
*                 _ _ _ ___|        |_______ _ _ _ 
*
*             
*****************************************************************************/
  
void I2C_Start(void)
{ 
    SDA_SET(1); 
    SCL_SET(1);  
    SDA_SET(0);                                 
}             



/*****************************************************************************
*
*     : I2C_Stop
*
*     : .
*
*    : .
*
*    :    STOP,    .
*		
*                                  |----------- - - - 
*   		                       | SCL
*		             _ _ _ ________| 
*
*                                       |-------- - - -
*                                       | SDA 
*                           _ _ _ ______|         
*  
*             
*****************************************************************************/

void I2C_Stop(void)
{ 
    SDA_SET(0);  
    SCL_SET(1);
    SDA_SET(1);
}
 


/*****************************************************************************
*
*     : I2C_WriteByte
*
*     :      
*                            (ACK). 1 = ACK, 0 = NACK.
*
*    :   .
*
*    :         ACK.
*
*            
*****************************************************************************/



unsigned char I2C_WriteByte(unsigned char data_byte)
{
  unsigned char i;
  unsigned char data_bit;        //       .
  unsigned char ack_status;      //   ACK.
  
    for(i=0; i<8; i++)           // 8- .
    { 
        data_bit = data_byte;          //        (         ).
        data_bit = (data_bit << i);    //         ,       i -
					         //    0,  1,2,3...    7. 

        data_bit = (data_bit & 0b10000000);   //     ,        ,
                                              //      i            ,
						          //      data_byte (   ,     I2C).  

        SCL_SET(0);              //       SDA, - SCL     SDA    
  				         //  (     START  STOP).
        if(data_bit == 0)        //      ...
            SDA_SET(0);          // SDA = 0.
              else               // (   ):
                SDA_SET(1);      //SDA = 1.
   
        SCL_SET(1);          //,          SDA  (    
    }				     //  ).
                             //         ACK  .             
    SCL_SET(0);              //    "" SCL.
   
    I2C_DDR.SDA_PIN = 0;	    //    SDA, .         SDA  .
      
    SCL_SET(1);                   //  SDA,           ACK  NACK.
   
    if(I2C_PORTIN.SDA_PIN == 0)   //    SDA  - ,   ACK.
        ack_status = ACK;
          else                    // ACK ,   NACK.
            ack_status = NACK;             
   
    SCL_SET(0);              // SCL  0,    ,        ACK  NACK  .
   
    I2C_DDR.SDA_PIN = 1;     // SDA    .

  return ack_status;         // ACK,      - NACK.
}

/*****************************************************************************
*
*     : I2C_ReadByte
*
*     :     I2C , .                           
*
*    : ACK       
*               NACK  .
*
*    :       ACK (NACK).       
*
*             
*****************************************************************************/

unsigned char I2C_ReadByte(unsigned char ack_status)
{ 
  unsigned char data_byte = 0;       //    .
  unsigned char i;
  
    I2C_DDR.SDA_PIN = 0;             //         SDA  .
  
    for(i=8; i>0; i--)               // 8- .
    {
        SCL_SET(1);                  //,  "", .      .

        if(I2C_PORTIN.SDA_PIN == 1)  //    SDA :
        {				       //        .       for
           data_byte |= (1<<(i-1));  //i    8,         - MSB. (         
        }				       //  0   7,    1   8,      (    
                                     //  I2C    " " (7,6,5...),      for c 8).
					       //     0 -    data_byte  .
       SCL_SET(0);                   // "" SCL,          .
    }
  					   //  ACK  .   ACK,         ACK
				         //()      (    I2C_ReadByte  ).
                                 // NACK,     .
    SCL_SET(0);                  //       SDA (   )   SCL  0.
  
    if(ack_status == ACK)        //  ACK, :
        SDA_SET(0);		   //SDA = 0.
          else		         // (NACK):
            SDA_SET(1);		   //SDA = 1.	
       
    SCL_SET(1);                  // SDA       ACK ( NACK).
    SCL_SET(0);			   //  SCL  0 (     ).
  
  return data_byte;              //    .
}


#endif
